package com.github.jfcloud.gene.file.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.lang.Assert;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.jfcloud.common.file.core.FileProperties;
import com.github.jfcloud.common.file.core.FileTemplate;
import com.github.jfcloud.gene.common.constant.WhetherEnum;
import com.github.jfcloud.gene.file.entity.FileInfo;
import com.github.jfcloud.gene.file.mapper.FileInfoMapper;
import com.github.jfcloud.gene.file.service.FileInfoService;
import com.github.jfcloud.gene.file.vo.FileVo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.io.InputStream;
import java.util.List;

@Slf4j
@Service
@RequiredArgsConstructor
public class FileInfoServiceImpl extends ServiceImpl<FileInfoMapper, FileInfo> implements FileInfoService {

    private final FileTemplate fileTemplate;

    private final FileProperties ossProperties;

    @Override
    public List<FileVo> listByRelateId(Long relateId) {
        List<FileInfo> list = list(new LambdaQueryWrapper<>(FileInfo.class)
                .eq(FileInfo::getRelateId, relateId)
                .eq(FileInfo::getDeleted, WhetherEnum.NO.getCode())
                .orderByAsc(FileInfo::getId));
        return BeanUtil.copyToList(list, FileVo.class);
    }

    public FileVo uploadFileWithFileName(InputStream fileInputStream, String fileName) {
        Assert.notNull(fileInputStream, "FileInputStream can not be null");

        FileVo vo = new FileVo();
        vo.setName(fileName);

        try {
            String bucketName = ossProperties.getBucketName();
            fileTemplate.removeObject(bucketName, fileName);
            fileTemplate.putObject(bucketName, fileName, fileInputStream);

            vo.setUrl(String.format("/admin/sys-file/%s/%s", bucketName, fileName));
        } catch (Exception e) {
            log.error("上传失败", e);
            Assert.isTrue(false, "附件生成上传失败");
        }
        return vo;
    }
}
