package com.github.jfcloud.gene.util;

import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.StrUtil;
import com.amazonaws.services.s3.model.S3Object;
import com.github.jfcloud.common.file.core.FileTemplate;
import com.github.jfcloud.common.util.DateUtil;
import com.github.jfcloud.common.util.UserUtil;
import com.github.jfcloud.gene.cache.UserIdNameCache;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.io.InputStream;
import java.util.Base64;
import java.util.Date;
import java.util.Map;
import java.util.Optional;

@Slf4j
@RequiredArgsConstructor
@Service
public class WordDataService {

    private final UserIdNameCache userIdNameCache;
    private final FileTemplate fileTemplate;

    /**
     * 填充文档创建人、创建时间、修改人、修改时间
     *
     * @param dataMap
     * @param createById 文档创建人id
     * @param createTime 文档创建时间
     */
    public void fillDocCreator(Map<String, Object> dataMap) {
        fillDocCreator(dataMap, UserUtil.getUserId(), new Date());
    }

    /**
     * 填充文档创建人、创建时间、修改人、修改时间
     *
     * @param dataMap
     * @param createById 文档创建人id
     * @param createTime 文档创建时间
     */
    public void fillDocCreator(Map<String, Object> dataMap, Long createById, Date createTime) {
        //文档创建人、创建时间
        String createBy = UserUtil.getUserId().equals(createById) ?
                UserUtil.getNickName() :
                Optional.ofNullable(userIdNameCache.getNicknameByUserId(createById))
                        .orElse("");
        dataMap.put("docCreator", createBy);
        dataMap.put("docCreatedTime", DateUtil.getISO8601Date(createTime));
        dataMap.put("docLastModifiedBy", UserUtil.getNickName());
        dataMap.put("docLastModifiedTime", DateUtil.getISO8601Date(new Date()));

        //当前年份、月份、日
        String[] split = DateUtil.formatDay(new Date()).split("-");
        dataMap.put("currentYear", split[0]);
        dataMap.put("currentMonth", split[1]);
        dataMap.put("currentDay", split[2]);
    }

    /**
     * 获取图片base64
     *
     * @param fileUrl 图片地址
     * @return base64字符串
     */
    public String getBase64(String fileUrl) {
        if (StrUtil.isBlank(fileUrl) || !fileUrl.startsWith("/admin/sys-file/")) {
            return "";
        }

        //存储桶图片
        String bucketName = fileUrl.substring(16);
        bucketName = bucketName.substring(0, bucketName.indexOf("/"));
        String filename = fileUrl.substring(fileUrl.lastIndexOf("/") + 1);

        try (S3Object s3Object = fileTemplate.getObject(bucketName, filename);
             InputStream inputStream = s3Object.getObjectContent()) {
            byte[] bytes = IoUtil.readBytes(inputStream);
            return Base64.getEncoder().encodeToString(bytes);

        } catch (Exception e) {
            log.error("图片读取异常: {}", e.getLocalizedMessage());
            return "";
        }
    }

}
